#!/usr/bin/perl -w

BEGIN { 
	our @INC;
	use File::Basename;
	use POSIX;

	my $prog_full_name = $0;
	my $prog_dir = dirname($prog_full_name);
	my $lib_dir = '';
	$prog_dir = getcwd if($prog_dir =~ /^\.$/);
#	$lib_dir = dirname($prog_dir);
	$lib_dir = $prog_dir;
	$lib_dir = "${lib_dir}/lib";
	unshift(@INC, $lib_dir) 
}

# use warnings; 
# use Search::Elasticsearch;
use strict;
use utf8;
binmode(STDOUT,':utf8');
binmode(STDIN,':utf8');
use Encode qw(decode encode);
use JSON::PP;
use Getopt::Long;
use Locale::gettext;
use Data::Dumper; 
use Sys::Hostname; 
use List::Util qw(min max); 
use File::stat; 
use File::Temp qw/ tempfile tempdir /;

my $prog_full_name = $0;
# my $work_dir = dirname($prog_full_name);

my $prog_name = basename($prog_full_name);
# $work_dir = getcwd if($work_dir =~ /^\.$/);

my $version = "1.0";
my $release = "1 (29.07.2024 12:33)";

my %set;
my %uset;

my $prog_name_b = $prog_name;
$prog_name_b =~ s/\.pl$//;

my $work_dir = "/usr/lib/$prog_name_b";
my $scripts_dir = "${work_dir}/scripts";
my $image_dir = "${work_dir}/image";

my $config_file = "${work_dir}/${prog_name_b}.conf";
my $drivers_json_file = "${work_dir}/drivers.json";

my $user_home_dir = $ENV{"HOME"};
my $user_set_dir = "${user_home_dir}/.config/$prog_name_b";
my $user_set_file = "${user_set_dir}/settings";
my $log_file = "${user_home_dir}/.config/${prog_name_b}.log";

my $debug = 0;
my $run_ask = 0;
my $run_auto = 0;
my $auto_start = 1;
my $dev_found = 0;
my $drv_found = 0;

my $d = Locale::gettext->domain("driver-manager");

my $install_drv_mess = $d->get("Install driver");
my $uninstall_drv_mess = $d->get("Delete driver");
my $found_drv_mess = $d->get("Drivers found for devices");
my $not_found_drv_mess = $d->get("No drivers found");
my $ok_button_mess = $d->get("Ok");
my $cancel_button_mess = $d->get("Cancel");
my $close_button_mess = $d->get("Close");
my $view_button_mess = $d->get("View");
my $not_run_button_mess = $d->get("Do not run again");
my $yad_title = $d->get("Driver Manager");

# ------------------------------------------------------------------------------
GetOptions( 'config-file=s' => \$config_file, 'c=s' => \$config_file,
			'ask' => \$run_ask, 'a' => \$run_ask,
			'auto' => \$run_auto,
			'version' => sub { PrintVersion(); }, 'v' => sub { PrintVersion(); },
			'help' => sub { Usage(0); }, 'h' => sub { Usage(0); } ) || Usage(1);

$set{'debug'} = $debug;
$set{'config_file'} = $config_file;
$set{'log_file'} = $log_file;
$set{'drivers_json_file'} = $drivers_json_file;
$set{'scripts_dir'} = $scripts_dir;
$set{'image_dir'} = $image_dir;

ParseConfig(\%set, $config_file);
ParseConfig(\%uset, $user_set_file) if(-e $user_set_file);

PrintLogMessage(Dumper(\%set));

$auto_start = $uset{'auto_start'} if(exists($uset{'auto_start'}));

$scripts_dir = $set{'scripts_dir'};
$image_dir = $set{'image_dir'};
$debug = $set{'debug'};
$log_file = $set{'log_file'};
$drivers_json_file = $set{'drivers_json_file'};

my $tmp_dir = tempdir( CLEANUP => 1 );
unless (-d $tmp_dir)
{
	PrintLogMessage("tempdir() error: $!");
	exit(1);
}

my $yad_sh_file = "${tmp_dir}/yad_run.sh";
my $yad_ask_sh_file = "${tmp_dir}/yad_run_ask.sh";

unless (-d $user_set_dir)
{
	unless (mkdir($user_set_dir))
	{
		PrintLogMessage("mkdir $user_set_dir error: $!");
		exit(1);
	}
}

my $yad_sh_fh;
unless (open($yad_sh_fh, '>utf8', "$yad_sh_file"))
{
	PrintLogMessage("$yad_sh_file open() error: $!");
	exit(1);	
}
my $yad_sh_ask_fh;
unless (open($yad_sh_ask_fh, '>utf8', "$yad_ask_sh_file"))
{
	PrintLogMessage("$yad_sh_file open() error: $!");
	exit(1);	
}

my $yad_key = int(rand(99999999));

print $yad_sh_fh "#!/bin/bash\n\n";
print $yad_sh_fh "export KEY=${yad_key}\n";
print $yad_sh_fh "ipcrm -M \$KEY &>/dev/null\n";
# print $yad_sh_fh "while true\ndo\n";

print $yad_sh_ask_fh "#!/bin/bash\n\n";

chmod 0755, $yad_sh_file;
chmod 0755, $yad_ask_sh_file;

my $json;
if (open(my $fh, "< $drivers_json_file"))
{
	local $/;
	$json = <$fh>;
	close($fh);
} else {
	PrintLogMessage( "Couldn't open configuration file $drivers_json_file : $!");
	exit(1);
}

# print "$json";
my $ds = JSON::PP->new->utf8->decode($json);

# print Dumper(\$drivers);

my $tabnum = 1;
my $driver_fields = '';
my @devs = $ds->{'devs'};
my $yad_tabs = '';
my $dev_names = '';

foreach my $dvs (@devs)
{
	foreach my $dev (@$dvs)
	{
		next unless(exists($dev->{'device'}));
		next unless(exists($dev->{'detect'}));
		next unless(exists($dev->{'drivers'}));

		my $dev_name = $dev->{'device'};

		my $detect_cmd = $dev->{'detect'};
		if($detect_cmd =~ /\.sh$/)
		{
			$detect_cmd = "${scripts_dir}/$detect_cmd";
			$detect_cmd = "$detect_cmd $dev_name $log_file";
		}

		next unless(RunCommand($detect_cmd, 0));

		$dev_found = 1;
		$yad_tabs = "$yad_tabs --tab=\"$dev_name\"";
		$dev_names = "${dev_names}\n${dev_name}\n ";

		my $dev_image = '';
		if(exists($dev->{'icon'}))
		{
			my $icon = "${image_dir}/$dev->{'icon'}";
			$dev_image = "--image=$icon";
		}

		$driver_fields = '';
		foreach my $drvs ($dev->{'drivers'})
		{
			foreach my $drv (@$drvs)
			{
				next unless(exists($drv->{'driver'}));
				next unless(exists($drv->{'detect'}));
				next unless(exists($drv->{'install'}));
				next unless(exists($drv->{'uninstall'}));
				my $drv_name = $drv->{'driver'};

				my $drv_detect_cmd = $drv->{'detect'};
				if($drv_detect_cmd =~ /\.sh$/)
				{
					$drv_detect_cmd = "${scripts_dir}/$drv_detect_cmd";
					$drv_detect_cmd = "$drv_detect_cmd $drv_name $log_file";
				}

				my $install_cmd = $drv->{'install'};
				if($install_cmd =~ /\.sh$/)
				{
					$install_cmd = "${scripts_dir}/$install_cmd";
					$install_cmd = "$install_cmd $drv_name $log_file $yad_key $prog_full_name";
				}

				my $uninstall_cmd = $drv->{'uninstall'};
				if($uninstall_cmd =~ /\.sh$/)
				{
					$uninstall_cmd = "${scripts_dir}/$uninstall_cmd";
					$uninstall_cmd = "$uninstall_cmd $drv_name $log_file $yad_key $prog_full_name";
				}

				if(RunCommand($drv_detect_cmd, 0))
				{
					$drv_found = 1;
					$driver_fields = "$driver_fields --field=\"$uninstall_drv_mess ${drv_name}:FBTN\" \"$uninstall_cmd\" ";
				} else {
							$driver_fields = "$driver_fields --field=\"$install_drv_mess $drv_name:FBTN\" \"$install_cmd\" ";
						}
			}

			print $yad_sh_fh "yad --plug=\$KEY --tabnum=$tabnum $dev_image --form $driver_fields &\n";
			$tabnum++;
		}
	} # foreach my $dev (@$dvs)
} # foreach my $dvs (@devs)

if($dev_found)
{
	print $yad_sh_fh "yad --center --window-icon=\"${image_dir}/driver-manager.svg\" --notebook --title=\"${yad_title}\" --key=\$KEY $yad_tabs --button=${close_button_mess}:1 \n";
	print $yad_sh_ask_fh "yad --center --title=\"${yad_title}\" --image=\"${image_dir}/driver-manager.svg\" --window-icon=\"${image_dir}/driver-manager.svg\" --text \"${found_drv_mess}:\n $dev_names\" --button=\"${view_button_mess}\":1 --button=${cancel_button_mess}:2 --button=\"${not_run_button_mess}\":3 \n";
} else {
			print $yad_sh_ask_fh "yad --center --title=\"${yad_title}\" --image=\"${image_dir}/driver-manager.svg\" --window-icon=\"${image_dir}/driver-manager.svg\" --text \"${not_found_drv_mess}\" --button=\"${close_button_mess}\":1 \n";
		}

if($run_auto && $auto_start && $dev_found)
{
	unless($drv_found)
	{
		my $rcode = RunCommand("/bin/bash $yad_ask_sh_file", 1);
		RunCommand("/bin/bash $yad_sh_file", 0) if($rcode == 1);
		$uset{'auto_start'} = 0 if($rcode == 3);
	}
} else {
			if($run_ask)
			{
				if($dev_found)
				{
					my $rcode = RunCommand("/bin/bash $yad_ask_sh_file", 1);
					RunCommand("/bin/bash $yad_sh_file", 0) if($rcode == 1);
					$uset{'auto_start'} = 0 if($rcode == 3);
				} else {
							RunCommand("/bin/bash $yad_ask_sh_file", 0);
						}
			} else {
						unless($run_auto)
						{
							if($dev_found)
							{
								RunCommand("/bin/bash $yad_sh_file", 0);
							} else {
										RunCommand("/bin/bash $yad_ask_sh_file", 0);
									}
						}
					}
		}
close($yad_sh_ask_fh);
close($yad_sh_fh);

unlink($yad_ask_sh_file);
unlink($yad_sh_file);

WriteSettings(\%uset, $user_set_file);

# ------------------------------------------------------------------------------
sub RunCommand
{
	my $command = shift;
	my $mode = shift;
	my $ret_code = 0;
	my @cmd_output = `$command 2>&1`;
	if($?)
	{
		$ret_code = $? >> 8;
		PrintLogMessage("\n Command \"$command\" failed:\n @cmd_output \n ret_code: $ret_code\n");
		if ($mode)
		{
			return $ret_code;
		} else { 
				return 0;
			}
	}
#	PrintLogMessage("$command :\n @cmd_output \n");
	if ($mode)
	{
		return $ret_code;
	} else { 
			return 1;
		}
}

# ------------------------------------------------------------------------------
sub StrsCastr
{
	for(@_)
	{
		return 0 unless(defined);
		chomp;
		s/^\s+//;
		s/\s+$//;
		return 0 unless(length);
	}
	return 1;
}

# ------------------------------------------------------------------------------
sub PrintLogMessage
{
	my $mess = shift;

	my $l_file  = $set{'log_file'};
	my $timestamp = localtime(time);
	my $fh;
	open($fh, ">> $l_file") or die "Couldn't open log file $l_file : $!\n";
	print $fh "$timestamp $mess\n";
	close($fh);
}

# ------------------------------------------------------------------------------
sub ParseConfig
{
	my $setting = shift;
	my $conf_file = shift;
	my $fh;

	unless (open($fh, "< $conf_file"))
	{
 		PrintLogMessage( "Couldn't open configuration file $conf_file : $!");
		exit(1);
	}

	while(<$fh>)
	{
		s/^\s*#.*$//;
		next unless(StrsCastr($_));
		my $line = $_;

		my ($var, $value) = split(/\s+/, $line, 2);
		next unless(StrsCastr($var, $value));

		$value =~ s/^"\s*//;
		$value =~ s/^'\s*//;

		$setting->{$var} = $value;

	} # while(<CONF>)

	close($fh);

} # sub ParseConfig

# ------------------------------------------------------------------------------
sub WriteSettings
{
	my $setting = shift;
	my $conf_file = shift;
	my $fh;

	unless (open($fh, "> $conf_file"))
	{
 		PrintLogMessage( "Couldn't open configuration file $conf_file : $!");
		exit(1);
	}
	my %st = %{$setting};
	foreach my $var (keys %st)
	{
		my $value = $st{$var};
		print $fh "$var $value \n";
	}

	close($fh);

} # sub ParseConfig

# ------------------------------------------------------------------------------
sub PrintVersion
{
	print "$prog_name version: ${version}-${release}.\n";
	exit;
}

# ------------------------------------------------------------------------------
sub Usage
{
	my $st = shift;

	print "Usage:\n";
	print "  $prog_name [ option ]\n";
	print "    option:\n";
	print "      -c | --config-file <file name>   - Configuration file name,default:\n\t\t\t\t\t   $config_file);\n";
	print "      -v | --version                   - Display version invormation and exit;\n";
	print "      -h | --help                      - Display this help and exit.\n";
	exit($st);
}
