#!/usr/bin/bash

DRIVER=$1
LOG_FILE=$2
KEY=$3
PROG=$4

exec 2>&1
exec >>$LOG_FILE

SCRIPT_DIR=`dirname $0`
. ${SCRIPT_DIR}/set.sh || exit 1

RUN_SCRIPT=`mktemp /tmp/drv_installXXXXX.sh`
chmod 755 $RUN_SCRIPT

SLEEP_SEC=1

if echo $DRIVER | grep '^kmod-focal-spi'; then
	cat >$RUN_SCRIPT <<EOF
#!/bin/bash
cat > /etc/yum.repos.d/inferit.repo << FOE
[inferit]
name=MSVSphere 9 - Inferit Hardware
baseurl=https://repo1.msvsphere-os.ru/vault/inferit/9/x86_64/
enabled=1
gpgcheck=1
countme=1
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-MSVSphere-9
metadata_expire=86400
enabled_metadata=1
FOE

dnf -y install $DRIVER libfprint-focal-spi --refresh
EOF
elif echo $DRIVER | grep '^akmod-focal-spi'; then
	SLEEP_SEC=2
	cat >$RUN_SCRIPT <<EOF
#!/bin/bash
cat > /etc/yum.repos.d/inferit.repo << FOE
[inferit]
name=MSVSphere 9 - Inferit Hardware
baseurl=https://repo1.msvsphere-os.ru/vault/inferit/9/x86_64/
enabled=1
gpgcheck=1
countme=1
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-MSVSphere-9
metadata_expire=86400
enabled_metadata=1
FOE

dnf -y install $DRIVER libfprint-focal-spi --refresh || exit 1
systemctl enable --now akmods || exit 1
# always run akmods and dracut at the end of updates
akmods || exit 1
dracut -f --regenerate-all
EOF
elif echo $DRIVER | grep '^kmod-'; then
		cat >$RUN_SCRIPT <<EOF
#!/bin/bash
dnf -y install $DRIVER --refresh
EOF
elif echo $DRIVER | grep '^akmod-'; then
	SLEEP_SEC=2
	cat >$RUN_SCRIPT <<EOF
#!/bin/bash
dnf -y install $DRIVER --refresh || exit 1
systemctl enable --now akmods || exit 1
# always run akmods and dracut at the end of updates
akmods || exit 1
dracut -f --regenerate-all 
EOF
elif echo $DRIVER | grep '^alsa-firmware'; then
cat >$RUN_SCRIPT <<EOF
#!/bin/bash

dnf -y install $DRIVER
sleep 5
exit 0
EOF
else
    echo "Error"
    exit 1
fi

${SCRIPT_DIR}/yad_progress.sh "$INST_DRV_MESS" $YAD_PROGRESS_PID_FILE 100 $SLEEP_SEC
pkexec $RUN_SCRIPT
RCODE=$?
kill `cat $YAD_PROGRESS_PID_FILE`
rm $YAD_PROGRESS_PID_FILE
rm $RUN_SCRIPT
if [ $RCODE -eq 0 ]; then
	yad $YAD_OPT --title="$YAD_TITLE" --text "$DRV_INSTALL_OK_MESS" --button "$RESTART_MESS":1 --button "$RESTART_LATER_MESS":2
else
	yad $YAD_OPT --title="$YAD_TITLE" --text "$DRV_INSTALL_ERR_MESS \nLog file: $LOG_FILE" --button "$CLOSE_BUTTON_MESS":2
fi
Quit $?
